#! /usr/bin/env python
# -*- Mode: Python -*-
# -*- coding: ascii -*-

__author__     = "Steven Pettit"
__date__       = "05 Aug 2020"
__copyright__  = "Steven Pettit"
__version__    = "1.0"
__maintainer__ = "Steven Pettit"
__email__      = "kryslin@tlhea.org"
__status__     = ""
__lwver__      = "11"

import os, sys, math, traceback
import lwsdk
import lwsdk.pris.modeler

lwpm = lwsdk.pris.modeler

class contractCPsel(lwsdk.ICommandSequence):
	def __init__(self, context):
		super(contractCPsel, self).__init__()

	def poly_scan(self, poly_list, poly_id):
		if poly_id.type == lwsdk.LWPOLTYPE_CURV: poly_list.append(poly_id.pol)

	def process(self, mod_command):
		try:
			#Contract CP selection
			lwpm.init(mod_command)
			lwpm.selmode(lwsdk.OPSEL_USER | lwsdk.OPSEL_MODIFY)
			lwpm.editbegin()
		
			meo = lwpm._local_me
			#get curve from modeler
			selp = []
			eor = meo.polyScan(meo.state, self.poly_scan, (selp,), lwsdk.OPLYR_FG)
						
			#if nothing selected, let's just exit...
			if lwpm.pointcount() == 0:
				return lwsdk.AFUNC_OK

			#find the first and last vertex that are selected...
			for p in selp:
				t = lwpm.polyinfo(p)
				l = t[1:]
				m = l[::-1]
				ll = len(l)
				#Build our initial state list
				sc = [int((meo.pointFlags(meo.state, v) != 0)) for v in l]
				tv = sum(sc)
				#If the list isn't empty...
				if tv != 0:
					cs = sc[::-1]
					lo = max(0, sc.index(1))
					hi = max(0, cs.index(1))
					if sc[lo] : meo.pntSelect(meo.state, l[lo], 0)
					if cs[hi] : meo.pntSelect(meo.state, m[hi], 0)
				
		except:
			print "We've had an error, closing down gracefully..."
			err = traceback.format_exc()
			print "Error : ", err 
		finally:
			lwpm.editend()
				
		return lwsdk.AFUNC_OK
			
ServerTagInfo = [( "contractCPsel", lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                 ( "Contract CP selection", lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH )]
ServerRecord = { lwsdk.CommandSequenceFactory("Contract CP selection", contractCPsel) : ServerTagInfo }

