#! /usr/bin/env python
# -*- Mode: Python -*-
# -*- coding: ascii -*-

__author__     = "Steven Pettit"
__date__       = "05 Aug 2020"
__copyright__  = "Steven Pettit"
__version__    = "1.0"
__maintainer__ = "Steven Pettit"
__email__      = "kryslin@tlhea.org"
__status__     = ""
__lwver__      = "11"

import os, sys, math, traceback
import lwsdk
import lwsdk.pris.modeler

lwpm = lwsdk.pris.modeler

class expandCPsel(lwsdk.ICommandSequence):
	def __init__(self, context):
		super(expandCPsel, self).__init__()
				
	def poly_scan(self, poly_list, poly_id):
		if poly_id.type == lwsdk.LWPOLTYPE_CURV: poly_list.append(poly_id.pol)
	
	def process(self, mod_command):
		try:
			#expand cp selection
			lwpm.init(mod_command)
			lwpm.selmode(lwsdk.OPSEL_DIRECT | lwsdk.OPSEL_MODIFY)
			lwpm.editbegin()
			meo = lwpm._local_me
			selv = lwpm.getpoints()
			#get all curves
			selp = []
			eor = meo.polyScan(meo.state, self.poly_scan, (selp,), lwsdk.OPLYR_FG)

			for p in selp:
				t = lwpm.polyinfo(p)
				l = t[1:]
				m = l[::-1]
				ll = len(l)
				#Build our initial state list
				sc = [int((meo.pointFlags(meo.state, v) != 0)) for v in l]
				#if the state of the curve = 0, then there is no selected points on it!
				tv = sum(sc)
				if tv != 0:
					#We have selected points!
					#The states of our vertices is in 0...ll-1
					#make a reversed list!
					cs = sc[::-1]
					lo = max(0, sc.index(1) - 1)
					hi = max(0, cs.index(1) - 1)
					if not sc[lo] : meo.pntSelect(meo.state, l[lo], 1)
					if not cs[hi] : meo.pntSelect(meo.state, m[hi], 1)

		except:
			print "We've had an error, closing down gracefully..."
			err = traceback.format_exc()
			print "Error : ", err 
		finally:
			lwpm.editend()
				
		return lwsdk.AFUNC_OK
			
ServerTagInfo = [( "expandCPsel", lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                 ( "Expand CP Selection", lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH )]
ServerRecord = { lwsdk.CommandSequenceFactory("Expand CP Selection", expandCPsel) : ServerTagInfo }

