#! /usr/bin/env python
# -*- Mode: Python -*-
# -*- coding: ascii -*-

__author__     = "Steven Pettit"
__date__       = "11 June 2020"
__copyright__  = "Steven Pettit"
__version__    = "1.0"
__maintainer__ = "Steven Pettit"
__email__      = "Kryslin@tlhea.org"
__status__     = ""
__lwver__      = "11"

import os, sys, math, traceback
import lwsdk
import lwsdk.pris.modeler

lwpm = lwsdk.pris.modeler

class extrudeCurves(lwsdk.ICommandSequence):
	def __init__(self, context):
		super(extrudeCurves, self).__init__()
		
		self.p_list = []
		self.v_list = []

	def norm_poly_scan1(self, poly_list, poly_id):
		if poly_id.flags & lwsdk.EDDF_SELECT and poly_id.type == lwsdk.LWPOLTYPE_CURV: poly_list.append(poly_id.pol)
		
	def norm_poly_scan2(self, poly_list, poly_id):
		if poly_id.type == lwsdk.LWPOLTYPE_CURV: poly_list.append(poly_id.pol)
				
	def norm_vert_scan1(self, vert_list, vert_id):
		if vert_id.flags & lwsdk.EDDF_SELECT: vert_list.append(vert_id.pnt)
		
	def norm_vert_scan2(self, vert_list, vert_id):
		vert_list.append(vert_id.pnt)
				
	def load_geo(self):
		mo = lwpm._local_me
		f = 0
		eor = mo.pointScan(mo.state, self.norm_vert_scan1, (self.v_list,), lwsdk.OPLYR_FG)
		if eor != lwsdk.EDERR_NONE:
			lwsdk.LWMessageFuncs().error("Point Scan Failed","")
	
		eor = mo.polyScan(mo.state, self.norm_poly_scan1, (self.p_list,), lwsdk.OPLYR_FG)
		if eor != lwsdk.EDERR_NONE:
			lwsdk.LWMessageFuncs().error("Point Scan Failed","")
				
		if not self.v_list:
			f = f | 1
			eor = mo.pointScan(mo.state, self.norm_vert_scan2, (self.v_list,), lwsdk.OPLYR_FG)
			if eor != lwsdk.EDERR_NONE:
				lwsdk.LWMessageFuncs().error("Point Scan Failed","")
			
		if not self.p_list:
			f = f | 2
			eor = mo.polyScan(mo.state, self.norm_poly_scan2, (self.p_list,), lwsdk.OPLYR_FG)
			if eor != lwsdk.EDERR_NONE:
				lwsdk.LWMessageFuncs().error("Point Scan Failed","")
				
		if not self.p_list: f = (f & 0b101) | 0b100
		if not self.v_list: f = f | 0b1000

		return f
		
	def process(self, mod_command):
		if not lwpm.init(mod_command):
			lwsdk.LWMessageFuncs().error("PRIS Not Initialized","")
			return lwsdk.AFUNC_OK

		whoops = False
		
		try:
			lwpm.selmode(lwsdk.OPSEL_DIRECT)
			lwpm.editbegin()
			res = self.load_geo()
			#We can work with curves selected, or points on curves, but no curves or no geometry is an exit condition.
			if res & 0b1100:
				lwsdk.LWMEssageFuncs().error('No usable geometry selected.')
				return lwsdk.AFUNC_OK
			#Selected Curves are the priority.
			selp =  []
			rtv = []
			if res & 0b0010:
				#We will need to get the point lists for all the curves.
				#Global Curves, Selected Polygons
			
				tmp = []
				for i,p in enumerate(self.p_list):
					tmp = lwpm.polyinfo(p)
					for j,v in enumerate(self.v_list):
						if v in tmp: rtv.append(p)

			#This is the expected result, with selected curves
			if (not res) | (res & 0b0001):
				for i,p in enumerate(self.p_list): rtv.append(p)
			
			for i,p in enumerate(rtv):
				tmp = lwpm.polyinfo(p)
				tl = len(tmp)-1
				lst = tmp[1:]
				newp = lwpm.addpoint(lwpm.pointinfo(tmp[tl]))
				lst.append(newp)
				selp.append(newp)
				lwpm.polypoints(p,lst)
				
		except:
			print("We've had an error, closing down gracefully...")
			traceback.print_exc()
			whoops = True

		finally:
			lwpm.editend()
			if not whoops:
				lwpm.selpoint('CLEAR','NPGT',0)
				lwpm.selpoint('SET','POINTID',selp)
				
		return lwsdk.AFUNC_OK
			
ServerTagInfo = [( "extrudeCurves", lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                 ( "Extrude Curves", lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH )]
ServerRecord = { lwsdk.CommandSequenceFactory("extrudeCurves", extrudeCurves) : ServerTagInfo }