#! /usr/bin/env python
# -*- Mode: Python -*-
# -*- coding: ascii -*-

import lwsdk.pris.modeler
import lwsdk.pris
import lwsdk
__author__ = "Steven Pettit"
__date__ = "24 June 2020"
__copyright__ = "Steven Pettit"
__version__ = "1.0"
__maintainer__ = "Steven Pettit"
__email__ = "kryslin@tlhea.org"
__status__ = ""
__lwver__ = "11"

import os
import sys
import math
import traceback

lwp = lwsdk.pris
lwpm = lwsdk.pris.modeler


class flattenPoints(lwsdk.ICommandSequence):
	def __init__(self, context):
		super(flattenPoints, self).__init__()

	def process(self, mod_command):
		if not lwpm.init(mod_command):
			raise Exception('Failed to Initialize PRIS')

		# region Set up some lists beforehand, used by the flattening algorithm
		self.zV = lwp.vector([0, 0, 0])
		self.mA = [lwp.vector([1, 0, 0]), lwp.vector([0, 1, 0]), lwp.vector([0, 0, 1]), lwp.vector([1, 1, 1])]
		self.mB = [lwp.vector([0, 1, 1]), lwp.vector([1, 0, 1]), lwp.vector([1, 1, 0]), lwp.vector([0, 0, 0])]
		self.tr = [self.zV, self.zV, self.zV, self.zV]
		self.user_axis = 0
		self.user_method = 0
		# endregion

		# region UI code goes here
		self.ui = lwsdk.LWPanels()
		self.panel = self.ui.create("Flatten Points")
		self.axis = self.panel.hchoice_ctl("axis", ["X", "Y", "Z", "Plane"])
		self.method = self.panel.hchoice_ctl("method", ["Min", "Max", "Average"])

		self.panel.align_controls_vertical([self.axis, self.method])
		self.panel.size_to_layout(5, 5)

		if self.panel.open(lwsdk.PANF_BLOCKING | lwsdk.PANF_CANCEL) == 0:
			self.ui.destroy(self.panel)
			return lwsdk.AFUNC_OK

		# Get our variables...
		self.user_axis = self.axis.get_int()
		self.user_method = self.method.get_int()
		# If axis = Plane, method doesn't matter.
		if self.user_axis == 3: self.user_method = 3
		# endregion

		try:
			self.cp = [0,0,0]
			lwpm.selmode(lwsdk.OPSEL_USER)
			lwpm.undogroupbegin()
			lwpm.editbegin()
			self.mo = lwpm._local_me
			self.pts = lwpm.getpoints()
			self.pcnt = len(self.pts)
			# A quick and dirty center point
			self.bb = lwpm.boundingbox()
			for j in range(0,3): self.cp[j] = (self.bb[0][j] + self.bb[1][j])/2.0
    
			self.mycp = lwpm.vector(self.cp)
			self.tr[0] = lwp.vector(self.bb[0])
			self.tr[1] = lwp.vector(self.bb[1])
			self.tr[2] = self.mycp
			self.tr[3] = self.zV
    
			if self.user_axis == 3:
				# We need to compute a few things, like vn and D
				# for out plane for the equations V dot P + D = 0
				self.ofs = int(max(2,round(self.pcnt * .33)))
				self.vA = lwp.normalize(lwpm.pointinfo(self.pts[0]) - self.mycp)
				self.vB = lwp.normalize(lwpm.pointinfo(self.pts[self.ofs]) - self.mycp)
				self.vN = lwp.normalize(lwp.cross3d(self.vA,self.vB))
				self.D = -(lwp.dot3d(self.vN,self.mycp))

			# Let's flatten some points!
			for i,v in enumerate(self.pts):
				self.tp = lwpm.pointinfo(v)
				if self.user_axis == 3:
					self.tD = lwp.dot3d(self.vN,self.tp) + self.D
					self.tp += self.vN * -self.tD
				else:
					self.tp = (self.mA[self.user_axis] * self.tr[self.user_method]) + (self.mB[self.user_axis] * self.tp)
				lwpm.pointmove(v,self.tp)
		except:
			print("We've had an error, closing down gracefully...")
			err = traceback.format_exc()
			print("Error : ", err)

		finally:
			lwpm.editend()
			lwpm.undogroupend()

		return lwsdk.AFUNC_OK
			
ServerTagInfo = [( "flattenPoints", lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                 ( "Flatten Points", lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH )]
ServerRecord = { lwsdk.CommandSequenceFactory("Flatten Points", flattenPoints) : ServerTagInfo }

