#! /usr/bin/env python
# -*- Mode: Python -*-
# -*- coding: ascii -*-

__author__     = "Steven Pettit"
__date__       = "23 June 2020"
__copyright__  = "Steven Pettit"
__version__    = "1.0"
__maintainer__ = "Steven Pettit"
__email__      = "kryslin@tlhea.org"
__status__     = ""
__lwver__      = "11"

import os, sys, math, traceback
import lwsdk
import lwsdk.pris.modeler

lwp = lwsdk.pris
lwpm = lwsdk.pris.modeler

class makeHair(lwsdk.ICommandSequence):
	def __init__(self, context):
		super(makeHair, self).__init__()
				
	def process(self, mod_command):
		#Callback for Basic Panel Events
		def callback(id, data):
			if id == self.root:	value = self.root.get_float()
			if id == self.hair: value = self.hair.get_float()
			if id == self.ptonsrf: value = self.ptonsrf.get_int()
			if id == self.newsurf: 
				value = self.newsurf.get_int()
				if value: 
					self.surfname.unghost()
				else:
					self.surfname.ghost()
					
			if id == self.surfname: value = self.surfname.get_str()	
			
		#Code for Plugin Starts Here
		try:
			if not lwpm.init(mod_command):
				lwsdk.LWMessageFuncs().error('PRIS not initialized.','')
				return lwsdk.AFUNC_OK
			
			self.above = lwp.recall('smp_MKHR_above',.010)
			self.below = lwp.recall('smp_MKHR_below',.005)
			self.onsrf = lwp.recall('smp_MKHR_onsurf',0)
			self.usesrf = lwp.recall('smp_MKHR_usesurf',0)
			self.srfnm = lwp.recall('smp_MKHR_surfnm','Default')
		
			self.ui    = lwsdk.LWPanels()
			self.panel = self.ui.create("Make Hair Curves 1.0")

			self.root = self.panel.dist_ctl("Root Depth")
			self.root.set_float(self.below)
			self.root.set_event(callback)
			self.hair = self.panel.dist_ctl("Hair Height")
			self.hair.set_float(self.above)
			self.hair.set_event(callback)
			self.ptonsrf = self.panel.bool_ctl("Point on Surface")
			self.ptonsrf.set_int(self.onsrf)
			self.ptonsrf.set_event(callback)
			self.newsurf = self.panel.bool_ctl("Use Surface Name")
			self.newsurf.set_int(self.usesrf)
			self.newsurf.set_event(callback)
			self.surfname = self.panel.str_ctl("Surface Name", 80)
			self.surfname.set_str(self.srfnm)
			self.surfname.set_event(callback)
			if not self.usesrf: self.surfname.ghost()

			self.panel.align_controls_vertical([self.root,self.hair,self.ptonsrf,self.newsurf,self.surfname])
			self.panel.size_to_layout(5, 5)

			if self.panel.open(lwsdk.PANF_BLOCKING | lwsdk.PANF_CANCEL) == 0:
				self.ui.destroy(self.panel)
				return lwsdk.AFUNC_OK # when cancel was clicked
				
			self.below = self.root.get_float()
			self.above = self.hair.get_float()
			self.onsrf = self.ptonsrf.get_int()
			self.usesrf = self.newsurf.get_int()
			if self.usesrf:
				self.srfnm = self.surfname.get_str()
			else:
				self.srfnm = "Default"
				
			#Do Modeler Stuff Here
			lwpm.selmode(lwsdk.OPSEL_USER)
			lwpm.undogroupbegin()
			lwpm.editbegin()
			mo = lwpm._local_me
			# Ok, we need to get a center point of each polygon, and a surface normal.
			self.p = lwpm.getpolygons()
			if not self.p:
				#No polygons are selected...
				lwsdk.LWMessageFuncs().error('No Polygons Selected.','')
				return lwsdk.AFUNC_OK	
			
			self.pl = []
			for h, pid in enumerate(self.p):
				cp = [0.0, 0.0, 0.0]
				tp = [0.0, 0.0, 0.0]
				pc = lwpm.polypointinfo(pid)
				tmp = lwpm.polyinfo(pid)
				pna = lwpm.polynormal(pid)
				pn = list(pna[1])
				t2 = [] 
				for i in range(1,pc+1):
					t2 = lwpm.pointinfo(tmp[i])
					for j in range(0,3): cp[j] += (t2[j] / pc)
				
				for j in range(0,3): tp[j] = cp[j] - (self.below * pn[j])
				self.pl.append(lwpm.addpoint(tp))
				if self.onsrf: self.pl.append(lwpm.addpoint(cp))
				for j in range(0,3): tp[j] = cp[j] + (self.above * pn[j])
				self.pl.append(lwpm.addpoint(tp))
				q = lwpm.addcurve(self.pl,self.srfnm)
				self.pl = []								
						
		except:
			print("We've had an error, closing down gracefully...")
			err = traceback.format_exc()
			print("Error : ", err)
			
		finally:
			lwpm.editend()
			lwpm.undogroupend()
			lwp.store('smp_MKHR_above',self.above)
			lwp.store('smp_MKHR_below',self.below)
			lwp.store('smp_MKHR_onsurf',self.onsrf)
			lwp.store('smp_MKHR_usesurf',self.usesrf)
			lwp.store('smp_MKHR_surfnm',self.srfnm)	
		
		return lwsdk.AFUNC_OK
			
ServerTagInfo = [( "makeHair", lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                 ( "Make Hair Curves", lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH )]
ServerRecord = { lwsdk.CommandSequenceFactory("Make Hair Curves", makeHair) : ServerTagInfo }

